/* $Id: eewrite.c,v 1.2 1996/03/26 22:50:08 james Exp $ */
/*
 * $Log: eewrite.c,v $
 * Revision 1.2  1996/03/26 22:50:08  james
 * Put the function return types on the line before the function's name.
 * Also split several over-long lines.
 *
 * Revision 1.1  1995/03/31 05:49:57  james
 * Initial revision
 *
 */

/* Utility to write to the Sound Galaxy NX Pro 16's EEPROM. */

#include <stdio.h>		/* fprintf() */
#include <stdlib.h>		/* strtoul() */
#include <string.h>		/* strchr() */
#include <ctype.h>		/* isspace() */
#include <getopt.h>		/*  getopt_long() */

#include "jy_sg.h"

#define PORT_IO_DEVICE "/dev/port" /* We may need setgid kmem. */

#define ASSERT(x) assert(x)

#define INT_BITS ( ( CHAR_BIT * sizeof(int) ) / sizeof(char) )


/*
 * static variables for command-line options.
 */
static int use_ascii;
static const char short_opts[] = "ab"; /* a for ASCII, b for Binary */
static struct option long_opts[] = 
{
  /* name    has_arg    int *flag,    val  */
  { "ascii",	0,	&use_ascii,	1 },
  { "binary",	0,	&use_ascii,	0 },

  { 0,		0,	0,		0 } /* end marker */
};
    
int
main(int argc, char *argv[])
{
  BYTE eeprom_location;
  BYTE eeprom_data[SG_NX_PRO_EEPROM_SIZE];
  
  /* Set the defaults */
  use_ascii =  1;

  while (1)
    {
      int optind;
      int c;

      c = getopt_long(argc, argv, short_opts, long_opts, &optind);
      if (c == -1)
	break;
      switch (c)		/* cope with short options */
	{
	case 'a':
	  use_ascii = 1;
	  break;
	case 'b':
	  use_ascii = 0;
	  break;
	}
    }      
  /* Done processing options. */
  fprintf(stderr, "%s input.\n", use_ascii ? "ASCII" : "Binary");
  fflush(stderr);

  if ( ! open_port_device(PORT_IO_DEVICE) )
    {
      fprintf(stderr, "Can\'t open Port I/O device, can\'t continue.\n");
      return 1;
    }
  
  /* Port IO device opened OK */

  /* Read the input. */
  memset(eeprom_data, 0, SG_NX_PRO_EEPROM_SIZE);
  
  if (use_ascii)
    {
#define MAXLINE 1000
      char line[MAXLINE];
      char *p;
      int line_num=0;
      
      
      eeprom_location = 0;
      while(1)
	{
	  /* Read a line */
	  if (NULL == fgets(line, MAXLINE, stdin))
	    {
	      if (feof(stdin))
		{
		  if (eeprom_location < SG_NX_PRO_EEPROM_SIZE)
		    {
		      fprintf(stderr,
			      "Unexpectedly early EOF on standard input.\n"
			      "Not writing the EEPROM.\n");
		      exit(1);
		    }
		  else
		    {
		      break;	/* Normal exit. */
		    }
		  
		}
	      else
		{
		  perror("standard input");
		  fprintf(stderr, "Not writing the EEPROM.\n");
		  exit(1);
		}
	    }
	  line_num++;
	  
	  /* Remove the comment from this single line, if any. */
	  p = strchr(line, '#');
	  if (p)
	    *p = '\0';
	  
	  /* Some lines may now be empty. */
	  if (strlen(line) == 0)
	    continue;
	  
	  /* Now digest the string */
	  p = line;
	  while(1)
	    {
	      unsigned long ulval;
	      char *last_p;

	      if (*p == '\0')
		break;
	      
	      last_p = p;
	      ulval = strtoul(p, &p, 16);

	      if (last_p == p)
		{
		  /* The conversion failed! */
		  if (*p == '\0')
		    {
		      break;	/* reached the end of the line */
		    }
		  else if (strchr("0123456789ABCDEFabcdef", *p) == 0)
		    {
		      fprintf(stderr,
			      "Unexpected non-hex-digits in input: "
			      "'%s' at line %d.\n"
			      "                                     ",
			      line, line_num);
		      while( (p--) > line)
			fputc(' ', stderr);
		      fprintf(stderr, "^ here\n");
		      return 1;
		    }
		  else if (p == line)		  /* Empty, no digits */
		    {
		      break;	/* finished with this line */
		    }
		}
	      else if (eeprom_location < SG_NX_PRO_EEPROM_SIZE)
		{
		  /* The conversion succeeded. */
		  eeprom_data[eeprom_location++] = (BYTE)ulval;
		}
	      else
		{
		  /* Successful conversion but too many hex numbers! */
		  fprintf(stderr,
			  "ASCII input has more hex values than needed.\n"
			  "Please check the input.  "
			  "The EEPROM has not been written.\n");
		  exit(1);
		}

	      while(*p && isspace(*p))
		p++;	/* skip over whitespace. */
	    }
	} /* read all the lines in the file */
    } /* ASCII */
  else
    {				
      for(eeprom_location=0;
	  eeprom_location < SG_NX_PRO_EEPROM_SIZE;
	  eeprom_location++)
	{
	  int chin = fgetc(stdin);
	  if (chin == EOF)
	    {
	      fprintf(stderr, "Unexpectedly early EOF on standard input.\n");
	      break;
	    }
	  eeprom_data[eeprom_location] = (BYTE)chin;
	}

      /* We have read as many values as we need; do we have more? */
      if (fgetc(stdin) != EOF)
	{
	  fprintf(stderr,
		  "There are more bytes in the binary "
		  "input than I think there are in the EEPROM.\n"
		  "Please check the input.  "
		  "The EEPROM has not been written.\n");
	  exit(1);
	}	  
    } /* BINARY */
  
  /* Write the data. */
  write_eeprom_block(eeprom_data, SG_NX_PRO_EEPROM_SIZE);
  fprintf(stderr, "\nWritten the EEPROM.\n");
  close_port_device();
  
  return 0;
}

